import { useFrame } from "@react-three/fiber";
import { forwardRef, useContext } from "react";
import { Vector3 } from "three";
import { vec3 } from "../../functions/Map";
import {
  SeismicSensor,
  TemperatureSensor,
  WindSensor,
} from "../../models/Data";
import RobotContext from "../../state/Robot";
import { robotState1 } from "../../data/Robot";

const Camera = forwardRef((props, ref: any) => {
  //Camera state
  const { cameraState } = useContext(RobotContext);

  useFrame((state) => {
    //Updating the camera position
    //Camera settings when position is set to Robot
    if (cameraState.position === "Robot") {
      // if we are in robot view we can either view the robot from this mode, which is on the same level as the robot
      if (cameraState.target === "Robot") {
        //Updating the lookAt each frame, to look at the robot's new position
        state.camera.lookAt(
          vec3(
            ref.current.robot.pos.x +
              15 * Math.cos(ref.current.robot.pos.r + Math.PI / 2),
            0.7,
            ref.current.robot.pos.y +
              15 * Math.sin(ref.current.robot.pos.r + Math.PI / 2)
          )
        );

        //Lerping the camera to the new position of the robot
        if (state.camera.position.y < -5) {
          state.camera.position.lerp(
            vec3(
              ref.current.robot.pos.x -
                20 * Math.cos(ref.current.robot.pos.r + Math.PI / 2),
              -5,
              ref.current.robot.pos.y -
                20 * Math.sin(ref.current.robot.pos.r + Math.PI / 2)
            ),
            0.04
          );
        } else {
          state.camera.position.lerp(
            vec3(
              ref.current.robot.pos.x -
                20 * Math.cos(ref.current.robot.pos.r + Math.PI / 2),
              -5,
              ref.current.robot.pos.y -
                20 * Math.sin(ref.current.robot.pos.r + Math.PI / 2)
            ),
            0.5
          );
        }

        //We can also view the robot in this mode which is slighty above the robot if we press C, (see the code in controller for event.code = "c")
      } else {
        //Updating the lookAt each frame, to look at the robot's new position
        if (
          state.camera.position.z > ref.current.robot.pos.y + 0.2 ||
          state.camera.position.z < ref.current.robot.pos.y - 0.2
        ) {
          state.camera.lookAt(
            vec3(ref.current.robot.pos.x, -0.7, ref.current.robot.pos.y)
          );
        }

        //Lerping the camera to the new position of the robot

        state.camera.position.lerp(
          vec3(ref.current.robot.pos.x, -80, ref.current.robot.pos.y),
          0.03
        );
      }

      //Setting default zoom and rotation value
      state.camera.up.set(0, -1, 0);
      state.camera.zoom = 2.0;
    }

    //Camera settings when position is set to God
    else if (cameraState.position === "God") {
      //Updating the lookAt to the origo position
      state.camera.lookAt(vec3(0, 0, 0));

      //Lerping the camera to the sky
      state.camera.position.lerp(vec3(0, -600, 0), 0.02);

      //Setting default zoom and rotation value
      state.camera.zoom = 0.75;
      state.camera.rotation.set(Math.PI / 2, 0, 0);
    }

    //Camera settings when position is set to Sensor
    else if (cameraState.position === "Sensor") {
      //Finding a sensor with a specific id, which has been choosen in the
      // camera select menu
      const foundSensor = ref.current.sensors.find(
        (s: WindSensor | TemperatureSensor | SeismicSensor) =>
          s.id === cameraState.sensorId
      );

      //Updating the lookAt each frame, to look at the robot's new position
      state.camera.lookAt(vec3(foundSensor.pos.x, -0.7, foundSensor.pos.y));

      //Lerping the camera to the sensor position
      state.camera.position.lerp(
        vec3(foundSensor.pos.x, -50, foundSensor.pos.y),
        0.1
      );

      //Setting default zoom and rotation value
      state.camera.zoom = 2.0;
    }

    //Camera settings when position is set to a Vector3
    else {
      const vector = cameraState.position as Vector3;
      //Updating the lookAt to the origo position
      state.camera.lookAt(vector);

      //Lerping the camera to the sky
      state.camera.position.lerp(vec3(vector.x, vector.y, vector.z), 0.1);

      //Setting default zoom and rotation value
      //state.camera.zoom = cameraState.zoom;
      state.camera.rotation.set(Math.PI / 2, 0, 0);
    }

    //Updating the projectionMatrix for the camera
    state.camera.updateProjectionMatrix();
  });

  return <mesh key={-1}></mesh>;
});

export default Camera;
