import React, { useContext, useState, forwardRef, useEffect } from "react";
import { Button, Row, Col } from "antd";
import { Item } from "../../../../models/Inventory";
import { Image } from "antd";
import InventoryContext from "../../../../state/InventoryCon";
import styles from "../styles/Overlay.module.scss";
import { InventoryItem } from "../../../../models/Data";
import StoneImg from "../../../../assets/items/Stone.png";
import AlienImg from "../../../../assets/items/Alien.png";
import WindSensorImg from "../../../../assets/items/WS.png";
import TempSensorImg from "../../../../assets/items/TS.png";
import SeismicSensorImg from "../../../../assets/items/SS.png";
import MQTTContext from "../../../../state/MQTT";
import ToggleAutopilot from "./ToggleAutopilot";
import RobotContext from "../../../../state/Robot";
import InstructionContext from "../../../../state/InstructionCon";
import { notification } from "antd";

interface UnitProps {
  item: InventoryItem | undefined;
  i: number;
  autoP: boolean;
  replay: boolean;
}

interface InfoProps {
  item: InventoryItem | undefined;
}

interface props {
  replay: boolean;
}
//information box, when each item is hovered over, this box with information is shown
const Info = ({ item }: InfoProps) => {
  return (
    <>
      {!item?.id ? (
        <></>
      ) : (
        <div className={styles.infoContainer}>
          <div className={styles.itemInfo}>
            <div>Item Id: {item.id}</div>
            <div>Item Mass: {item.mass}</div>
            <div>Item Radius: {item.radius}</div>
            <div>Item Type: {item.type}</div>
          </div>
        </div>
      )}
    </>
  );
};

//single inventory square = one unit
const InventoryUnit = ({ item, i, autoP, replay }: UnitProps) => {
  const [showInfo, setShowInfo] = React.useState(false);
  const { removeFromInventoryContents } = useContext(InventoryContext);
  const { send, config } = useContext(MQTTContext);
  const { setCameraState, setInteractionMode } = useContext(RobotContext);
  const { setCurrentObjectId } = useContext(InstructionContext);

  //if an item is clicked in inventory, send drop command to simulator
  function clickItem(i: number) {
    if (autoP === false) {
      send(`sim/${config.id}/in/control`, {
        type: "drop",
        data: { index: i - 1 },
      });
    } else {
      if (!item) return;
      let tempCameraState = {} as any;
      tempCameraState.target = "Origo";
      tempCameraState.position = "God";
      setCameraState(tempCameraState);
      setInteractionMode("selectGrid");
      setCurrentObjectId(item.id);
      console.log(item.id, i);
    }
  }
  return (
    <div
      style={{
        borderRadius: 15,
        height: 60,
        width: 60,
        background: "rgba(0, 0, 0, 0.6)",
        margin: 0,
      }}
    >
      {!item ? (
        <></>
      ) : (
        <>
          {/* hover functionality on mouse enter and leave*/}
          {showInfo && <Info item={item} />}
          <Button
            onMouseEnter={() => setShowInfo(true)}
            onMouseLeave={() => setShowInfo(false)}
            onClick={() => {
              if (replay) return;
              clickItem(i);
            }}
            style={{
              height: "100%",
              width: "100%",
              margin: 0,
              padding: 0,
              borderRadius: 15,
              backgroundColor: "rgba(0, 0, 0, 0)",
            }}
          >
            {/* images for each item, check what item is in inventory and display correct image*/}
            <Image
              style={{ borderRadius: 15, maxHeight: "100%" }}
              src={
                item.type === "Stone"
                  ? StoneImg
                  : item.type === "Alien"
                  ? AlienImg
                  : item.type === "Wind Sensor"
                  ? WindSensorImg
                  : item.type === "Temperature Sensor"
                  ? TempSensorImg
                  : item.type === "Seismic Sensor"
                  ? SeismicSensorImg
                  : ""
              }
              preview={false}
              height={"100%"}
              width={"100%"}
            />
          </Button>
        </>
      )}
      <div style={{ visibility: "hidden" }} className={styles.quantity}>
        {1 /*item?.quantity*/} {i}
      </div>
    </div>
  );
};

const Inventory = forwardRef(({ replay }: props, ref: any) => {
  const [inventoryContents, setInventoryContents] = useState<InventoryItem[]>(
    []
  );
  const [sendErrorFull, setSendErrorFull] = React.useState(false);
  const [autopilot, setAutopilot] = React.useState(false);
  const { send, config } = useContext(MQTTContext);
  const inventorySize = 5;

  //get information from payload sent by simulator, every second
  useEffect(() => {
    const interval = setInterval(() => {
      if (ref.current.robot.storage.length <= inventorySize) {
        setSendErrorFull(false);
        setInventoryContents(ref.current.robot.storage);
      } else if (
        ref.current.robot.storage.length > inventorySize &&
        sendErrorFull === false
      ) {
        setSendErrorFull(true);
      }
      setAutopilot(ref.current.autopilot);
    }, 1000);

    return () => clearInterval(interval);
  }, [ref]);

  //if inventory is full, send drop command to simulator to drop latest item
  useEffect(() => {
    if (sendErrorFull === true) {
      notification.open({
        message: "Error",
        description: "Inventory full, can't pickup item",
        type: "error",
      });
      send(`sim/${config.id}/in/control`, {
        type: "drop",
        data: { index: inventorySize },
      });
    }
  }, [sendErrorFull]);

  return (
    <div
      style={{
        padding: 5,
        display: "flex",
        alignItems: "center",
        flexDirection: "row",
        justifyContent: "space-between",
        background: "rgba(0, 0, 0, 0)",
        borderRadius: 15,
      }}
    >
      <div>
        {/* antd row and column, for loop creates inventory spaces based on the size wanted, not hard coded*/}
        <Row gutter={[10, 10]}>
          {Array.from(
            {
              length: inventorySize,
            },
            (_, i) => i + 1
          ).map((n) => (
            <Col key={n}>
              <InventoryUnit
                item={
                  inventoryContents[n - 1]
                    ? inventoryContents[n - 1]
                    : undefined
                }
                i={n}
                autoP={autopilot}
                replay={replay}
              />
            </Col>
          ))}
        </Row>
      </div>
    </div>
  );
});

export default Inventory;

// git checkout Kristiyan
