export interface Messages {
  msgs: Message[];
  time: number;
}

export interface Message {
  data:
    | RobotData
    | WindSensor
    | TemperatureSensor
    | SeismicSensor
    | RobotPos
    | LidarObject[]
    | GeneralInformation
    | Position2
    | RobotArm;
  type:
    | "Proximity Sensor"
    | "Robot"
    | "Pickup Sensor"
    | "Temperature Sensor"
    | "Wind Sensor"
    | "Seismic Sensor"
    | "lidar"
    | "RobotPos"
    | "objects"
    | "info"
    | "path"
    | "arm";
}

export interface RobotArm {
  arm1_jointAngle: number;
  arm1_jointSpeed: number;
  arm1_jointmotorSpeed: number;
  arm2_jointAngle: number;
  arm2_jointSpeed: number;
  arm2_jointmotorSpeed: number;
  arm3_jointAngle: number;
  arm3_jointSpeed: number;
  arm3_jointmotorSpeed: number;
  arm_opened: boolean;
}

export interface WindSensor {
  id: number;
  pos: Position2;
  wind_vec: Position2;
}

export interface TemperatureSensor {
  id: number;
  pos: Position2;
  temp: number;
}

export interface SeismicSensor {
  id: number;
  pos: Position2;
  shake_val: number;
}

export interface RobotData {
  battery: number;
  in_shadow: boolean;
  pos: Position;
  storage: string[];
}

export interface RobotPos {
  pos: Position;
}

export interface GeneralInformation {
  satelliteImageScaleFactor: number;
  shadowFrontierR: number;
  shadowFrontierX: number;
  shadowFrontierY: number;
}

export interface SituationReport {
  mleElements: MleElement[];
  gridElements: GridElement[];
  gridWidth: number;
  gridHeight: number;
  gridSize: number;
}

export interface Position {
  r: number;
  x: number;
  y: number;
}

export interface Position2 {
  x: number;
  y: number;
}

export interface Object {
  id: number;
  mass: number;
  name: string;
  pos: Position;
}

export interface InventoryItem {
  id: number;
  mass: number;
  radius: number;
  type: string;
}

export interface LidarPosition {
  x: number;
  y: number;
  r: number; //RADIUS
}

export interface LidarObject {
  id: string;
  type: string;
  data: LidarPosition;
}

export interface Conclusion {
  status: number; //0=success 1=failed
  tornado: number; //0=no 1=yes
  earthquake: number; //0=no 1=yes
  vulcano: number; //0=none 1=dormant 2=erupt
}

export interface MleState {
  state: string;
  value: number;
}

//MLE Element that gives information about a grid and its position,
// and the most likely event
export interface MleElement {
  position: Position2;
  alien: MleState;
  earthquake: MleState;
  tornado: MleState;
  volcano: MleState;
}

export interface VolcanoGrid {
  none: number;
  dormant: number;
  erupt: number;
}

//Grid Element that gives information about a grid and its position,
// and all the probabilities for all the events
export interface GridElement {
  position: Position2;
  alien: number;
  earthquake: number;
  tornado: number;
  volcano: VolcanoGrid;
}

//Interfaces used by sit. report for sending data
export interface SitMessage {
  data_grid: GridElement[];
  data_mle: MleElement[];
  gridWidth: number;
  gridHeight: number;
}

export interface Task {
  id: number;
  type: string;
  data: any;
}

export interface MapScreenData {
  robot: RobotData;
  conclusion: Conclusion;
  lidarObjects: LidarObject[];
  satelitteImage: string;
  generalInformation: GeneralInformation;
  sensors: (WindSensor | TemperatureSensor | SeismicSensor)[];
  autopilot: boolean;
  plan: any;
  motionPath: Position2[];
  hoveredCoordinate: Position2;
  record: boolean;
  robotarm: RobotArm;
  situationReport: SituationReport;
}

export interface DashboardScreenData {
  robot: RobotData;
  conclusion: Conclusion;
  sensors: (WindSensor | TemperatureSensor | SeismicSensor)[];
  autopilot: boolean;
  plan: any;
}

//init state for mapScreenData
export const initMapScreenData = {
  robot: {
    battery: 100,
    in_shadow: false,
    pos: { r: 0, x: 0, y: 0 },
    storage: [],
  },
  conclusion: {
    status: 1,
    tornado: 0,
    earthquake: 0,
    vulcano: 0,
  },
  lidarObjects: [],
  satelitteImage: "",
  generalInformation: {
    satelliteImageScaleFactor: 1,
    shadowFrontierR: 1,
    shadowFrontierX: 500,
    shadowFrontierY: 500,
  },
  sensors: [],
  autopilot: true,
  plan: {
    "0": {
      type: "goto",
      data: { x: -69.65708923339844, y: 134.83541870117188 },
    },
    "1": { type: "pickup", data: {} },
    "2": {
      type: "goto",
      data: { x: 349.41900634765625, y: 377.8659973144531 },
    },
    "3": { type: "pickup", data: {} },
    "4": { type: "goto", data: { x: 100, y: 100 } },
    "5": { type: "drop", data: { index: 1 } },
    "6": {
      type: "goto",
      data: { x: -141.46917724609375, y: -367.6352233886719 },
    },
    "7": { type: "pickup", data: {} },
  } as any,
  motionPath: [],
  hoveredCoordinate: { x: 0, y: 0 },
  record: false,
  robotarm: {
    arm1_jointAngle: 0,
    arm1_jointSpeed: 0,
    arm1_jointmotorSpeed: 0,
    arm2_jointAngle: 0,
    arm2_jointSpeed: 0,
    arm2_jointmotorSpeed: 0,
    arm3_jointAngle: 0,
    arm3_jointSpeed: 0,
    arm3_jointmotorSpeed: 0,
    arm_opened: false,
  },
  situationReport: {
    mleElements: [
      {
        position: { x: 0, y: 0 },
        alien: { state: "no", value: 1 },
        earthquake: { state: "no", value: 1 },
        tornado: { state: "no", value: 1 },
        volcano: { state: "dormant", value: 1 },
      },
      {
        position: { x: 10, y: -40 },
        alien: { state: "no", value: 1 },
        earthquake: { state: "no", value: 1 },
        tornado: { state: "no", value: 1 },
        volcano: { state: "dormant", value: 1 },
      },
      {
        position: { x: 120, y: 150 },
        alien: { state: "no", value: 1 },
        earthquake: { state: "no", value: 1 },
        tornado: { state: "no", value: 1 },
        volcano: { state: "dormant", value: 1 },
      },
    ],
    gridElements: [],
    gridWidth: 10,
    gridHeight: 10,
    gridSize: 15,
  },
} as MapScreenData;

//init state for mapScreenData
export const initReplayScreenData = {
  robot: {
    battery: 100,
    in_shadow: false,
    pos: { r: 0, x: 0, y: 0 },
    storage: [],
  },
  conclusion: {
    status: 1,
    tornado: 0,
    earthquake: 0,
    vulcano: 0,
  },
  lidarObjects: [],
  satelitteImage: "",
  generalInformation: {
    satelliteImageScaleFactor: 1,
    shadowFrontierR: 1,
    shadowFrontierX: 500,
    shadowFrontierY: 500,
  },
  sensors: [],
  autopilot: true,
  plan: {
    "0": {
      type: "goto",
      data: { x: -69.65708923339844, y: 134.83541870117188 },
    },
    "1": { type: "pickup", data: {} },
    "2": {
      type: "goto",
      data: { x: 349.41900634765625, y: 377.8659973144531 },
    },
    "3": { type: "pickup", data: {} },
    "4": { type: "goto", data: { x: 100, y: 100 } },
    "5": { type: "drop", data: { index: 1 } },
    "6": {
      type: "goto",
      data: { x: -141.46917724609375, y: -367.6352233886719 },
    },
    "7": { type: "pickup", data: {} },
  } as any,
  motionPath: [],
  hoveredCoordinate: { x: 0, y: 0 },
  record: false,
  robotarm: {
    arm1_jointAngle: 0,
    arm1_jointSpeed: 0,
    arm1_jointmotorSpeed: 0,
    arm2_jointAngle: 0,
    arm2_jointSpeed: 0,
    arm2_jointmotorSpeed: 0,
    arm3_jointAngle: 0,
    arm3_jointSpeed: 0,
    arm3_jointmotorSpeed: 0,
    arm_opened: false,
  },
  situationReport: {
    mleElements: [],
    gridElements: [],
    gridWidth: 10,
    gridHeight: 10,
    gridSize: 15,
  },
} as MapScreenData;

//init state for dashboardScreenData
export const initDashboardScreenData = {
  robot: {
    battery: 100,
    in_shadow: false,
    pos: { r: 0, x: 0, y: 0 },
    storage: [],
  },
  conclusion: {
    status: 1,
    tornado: 0,
    earthquake: 0,
    vulcano: 0,
  },
  sensors: [],
  autopilot: true,
  plan: {
    "0": {
      type: "goto",
      data: { x: -69.65708923339844, y: 134.83541870117188 },
    },
    "1": { type: "pickup", data: {} },
    "2": {
      type: "goto",
      data: { x: 349.41900634765625, y: 377.8659973144531 },
    },
    "3": { type: "pickup", data: {} },
    "4": { type: "goto", data: { x: 100, y: 100 } },
    "5": { type: "drop", data: { index: 1 } },
    "6": {
      type: "goto",
      data: { x: -141.46917724609375, y: -367.6352233886719 },
    },
    "7": { type: "pickup", data: {} },
  } as any,
} as DashboardScreenData;
