import { Route, Routes } from "react-router-dom";
import ReplayScreen from "../replay/ReplayScreen";
import Layout from "./components/Layout";
import RobotContext, { RobotContextProvider } from "../state/Robot";
import MapScreen from "../map/MapScreen";
import { useContext, useEffect, useRef } from "react";
import { ConfigProvider, theme } from "antd";
import { SupabaseContextProvider } from "../state/Supabase";
import { OverlayContextProvider } from "../state/FullScreenOverlayLayout";
import DashboardScreen from "../dashboard/DashboardScreen";
import { InventoryContextProvider } from "../state/InventoryCon";
import { InstructionContextProvider } from "../state/InstructionCon";
import { Message } from "paho-mqtt";
import { decode } from "messagepack";
import Pako from "pako";
import {
  DashboardScreenData,
  GeneralInformation,
  initDashboardScreenData,
  initMapScreenData,
  LidarObject,
  MapScreenData,
  Messages,
  RobotData,
  RobotPos,
  SeismicSensor,
  TemperatureSensor,
  WindSensor,
  Message as MsgStruct,
  Position2,
  Conclusion,
  RobotArm,
  GridElement,
  SitMessage,
  MleElement,
  SituationReport,
} from "../models/Data";
import MQTTContext from "../state/MQTT";
import { MapContextProvider } from "../state/Map";
import JoystickScreen from "../mobile/joystick/JoystickScreen";
import MobileLandingScreen from "../mobile/MobileLandingScreen";
import { uploadDemo } from "../functions/Supabase";
import moment from "moment";
import { Frame, FrameData } from "../models/Replay";
import LoadingScreen from "../components/LoadingScreen";
import { jsonEscape } from "../helpers/Data";

const Navigation = () => {
  //Importing client and different set functions from contexts
  const { config, client, send } = useContext(MQTTContext);
  const {
    updateSeismic,
    updateTemperature,
    updateWindSpeed,
    setRobotArmArray,
    setRobotArmOpen,
  } = useContext(RobotContext);

  //Creating refs for the data to be stored
  const mapScreenData = useRef<MapScreenData>(initMapScreenData);
  const dashboardScreenData = useRef<DashboardScreenData>(
    initDashboardScreenData
  );

  //Creating variables to hold recorded data
  //frame count
  const frameCount = useRef<number>(0);
  //total frame array com.
  let frameArray = useRef<Frame[]>([]);
  //currentFrameArray
  let currentFrameDataArray = useRef<FrameData[]>([]);

  //initilize prev. states for each of the data properties
  let prevRobotData = {
    battery: 100,
    in_shadow: false,
    pos: { r: 0, x: 0, y: 0 },
    storage: [],
  } as RobotData;

  let prevConclusion = {
    status: 1,
    tornado: 0,
    earthquake: 0,
    vulcano: 0,
  } as Conclusion;
  let prevLidarObjects = [] as LidarObject[];
  let prevSatelitteImage = "" as string;
  let prevGeneralInformation = {
    satelliteImageScaleFactor: 1,
    shadowFrontierR: 1,
    shadowFrontierX: 1,
    shadowFrontierY: 1,
  } as GeneralInformation;
  let prevSensors = [] as (WindSensor | TemperatureSensor | SeismicSensor)[];
  let prevAutopilot = true as boolean;
  let prevPlan = {} as any;
  let prevMotionPath = [] as Position2[];
  let prevSituationReport = {
    mleElements: [],
    gridElements: [],
    gridHeight: 20,
    gridWidth: 20,
  } as SituationReport;

  let prevRobotArm = {
    arm1_jointAngle: 0,
    arm1_jointSpeed: 0,
    arm1_jointmotorSpeed: 0,
    arm2_jointAngle: 0,
    arm2_jointSpeed: 0,
    arm2_jointmotorSpeed: 0,
    arm3_jointAngle: 0,
    arm3_jointSpeed: 0,
    arm3_jointmotorSpeed: 0,
    arm_opened: true,
  } as RobotArm;

  //Interval that saves frames and uploades a demo if the record mode is ON
  useEffect(() => {
    const interval = setInterval(() => {
      if (mapScreenData.current.record) {
        //Creating a new frame
        const newFrame = {
          id: frameCount.current,
          data: currentFrameDataArray.current,
          created_at: moment().format("YYYY-MM-DD HH:mm:ss"),
        } as Frame;

        //add the frame data to buffer and reset current frame data
        frameArray.current.push(newFrame);
        currentFrameDataArray.current = [];

        //increase frame count
        frameCount.current = frameCount.current + 1;
      } else {
        if (frameArray.current.length > 0) {
          uploadDemo(frameArray.current);
          frameCount.current = 0;
          frameArray.current = [];
        }
      }
    }, 30);
    return () => clearInterval(interval);
  }, []);

  //Creating callback functions to update the ref
  const switchAutopilot = () => {
    //Switches the autopilot
    mapScreenData.current.autopilot = !mapScreenData.current.autopilot;
    dashboardScreenData.current.autopilot = !mapScreenData.current.autopilot;

    //Send the data to a autopilot topic
    send("ui/out/autopilot", {
      type: "autopilot",
      data: mapScreenData.current.autopilot,
    });

    //If in record mode
    if (mapScreenData.current.record) {
      //Check if frame already has the info
      if (
        currentFrameDataArray.current.filter((d) => d.type === "Autopilot")
          .length < 1
      ) {
        //Check if robot pos. has been updated
        if (
          JSON.stringify(prevAutopilot) !==
          JSON.stringify(mapScreenData.current.autopilot)
        ) {
          //Adding the update to the current frame data
          const newFrameData = {
            type: "Autopilot",
            data: mapScreenData.current.autopilot,
          } as FrameData;
          currentFrameDataArray.current.push(newFrameData);
          //Update prev. state
          prevAutopilot = mapScreenData.current.autopilot;
        }
      }
    }
  };

  //Creating callback functions to update the ref
  const switchRecord = () => {
    //Switches the record
    mapScreenData.current.record = !mapScreenData.current.record;
  };

  //Creating callback functions to update the ref
  const setHoveredCoordinate = (position: Position2) => {
    //Sets the hovered coordinate
    mapScreenData.current.hoveredCoordinate = position;
  };

  //Check so that the MQTT client is defined
  if (!client) return <LoadingScreen />;

  //Setting the function to handle message arrive from the server
  client.onMessageArrived = onMessageArrived;

  //On message arrive from MQTT server
  function onMessageArrived(message: Message) {
    const compressedMsg = message.payloadBytes;
    try {
      //Here we update the satelitte image, it changes for each mission
      if (message.destinationName === `sim/${config.id}/out/image`) {
        console.log("Getting img.");
        const blob = new Blob([compressedMsg]);
        const srcBlob = URL.createObjectURL(blob);
        console.log(srcBlob);
        mapScreenData.current.satelitteImage = srcBlob;
        return;
      }

      //Decompress the message with zlib and decode it with messagepack
      const decompressedMsg = decode(Pako.inflate(compressedMsg));
      const jsonString = JSON.stringify(decompressedMsg);

      //This topic holds the robot's positon
      if (message.destinationName === `sim/${config.id}/out/robotpos`) {
        //Parse the message
        const jsonObject = JSON.parse(jsonString) as Messages;
        const { msgs } = jsonObject;
        const data = msgs[0].data as RobotPos;

        //Update the refs
        mapScreenData.current.robot.pos.r = data.pos.r;
        mapScreenData.current.robot.pos.x = data.pos.x;
        mapScreenData.current.robot.pos.y = data.pos.y;
        dashboardScreenData.current.robot.pos = mapScreenData.current.robot.pos;

        //If in record mode
        if (mapScreenData.current.record) {
          //Check if frame already has the info
          if (
            currentFrameDataArray.current.filter((d) => d.type === "Robot")
              .length < 1
          ) {
            //Check if robot pos. has been updated
            if (
              JSON.stringify(prevRobotData.pos) !== JSON.stringify(data.pos)
            ) {
              //Creating a new temp. object that will hold the updated robot position
              let newRobotData = prevRobotData;
              newRobotData.pos = data.pos;

              //Adding the update to the current frame data
              const newFrameData = {
                type: "Robot",
                data: newRobotData,
              } as FrameData;
              currentFrameDataArray.current.push(newFrameData);

              //Update prev. state
              prevRobotData.pos = data.pos;
            }
          }
        }
      }

      //This topic holds the sensed lidar objects
      else if (message.destinationName === "sit/out/lidar_ui") {
        mapScreenData.current.lidarObjects = [];
        const jsonObject = JSON.parse(jsonString) as LidarObject[];
        mapScreenData.current.lidarObjects = jsonObject;

        //If in record mode
        if (mapScreenData.current.record) {
          //Check if frame already has the info
          if (
            currentFrameDataArray.current.filter(
              (d) => d.type === "Lidar Objects"
            ).length < 1
          ) {
            //Check if robot pos. has been updated
            if (
              JSON.stringify(prevLidarObjects) !== JSON.stringify(jsonObject)
            ) {
              //Adding the update to the current frame data
              const newFrameData = {
                type: "Lidar Objects",
                data: jsonObject,
              } as FrameData;
              currentFrameDataArray.current.push(newFrameData);
              //Update prev. state
              prevLidarObjects = jsonObject;
            }
          }
        }
      }

      //This topic holds the sensed lidar objects
      else if (message.destinationName === "sit/out/grids") {
        const jsonObject = JSON.parse(jsonString);
        const stringObject = jsonObject as string;
        const { data_grid, data_mle, gridWidth, gridHeight } = JSON.parse(
          stringObject
        ) as SitMessage;
        mapScreenData.current.situationReport.mleElements = data_mle;
        mapScreenData.current.situationReport.gridElements = data_grid;
        mapScreenData.current.situationReport.gridWidth = gridWidth;
        mapScreenData.current.situationReport.gridHeight = gridHeight;
        if (mapScreenData.current.record) {
          //Check if frame already has the info
          if (
            currentFrameDataArray.current.filter(
              (d) => d.type === "Situation Report"
            ).length < 1
          ) {
            //Check if grid conclusion has been updated
            if (
              JSON.stringify(prevSituationReport) !== JSON.stringify(jsonObject)
            ) {
              //Adding the update to the current frame data
              const newFrameData = {
                type: "Situation Report",
                data: {
                  gridElements: data_grid,
                  mleElements: data_mle,
                  gridHeight: gridHeight,
                  gridWidth: gridWidth,
                },
              } as FrameData;
              currentFrameDataArray.current.push(newFrameData);
              //Update prev. state
              prevSituationReport = jsonObject;
            }
          }
        }
      }

      //This topic holds the proximity sensors and their sensed objects
      else if (message.destinationName === `sim/${config.id}/out/general`) {
        const jsonObject = JSON.parse(jsonString) as Messages;
        const { msgs } = jsonObject;

        for (let i = 0; i < msgs.length; i++) {
          if (msgs[i].type === "Robot") {
            const data = msgs[i].data as RobotData;
            mapScreenData.current.robot.battery = data.battery;
            mapScreenData.current.robot.storage = data.storage;
            mapScreenData.current.robot.in_shadow = data.in_shadow;

            dashboardScreenData.current.robot.battery = data.battery;
            dashboardScreenData.current.robot.storage = data.storage;
            dashboardScreenData.current.robot.in_shadow = data.in_shadow;

            //If in record mode
            if (mapScreenData.current.record) {
              //Update prev. state
              //Here the update won't happend until the robot position changes
              //since they both update the same object, the one that updates
              //with higher frequency (the pos. in this case) will handle the
              //frame update
              prevRobotData = data;
            }
          }
        }
      }

      //This topic holds general map information
      else if (message.destinationName === `sim/${config.id}/out/info`) {
        const jsonObject = JSON.parse(jsonString) as Messages;

        const { msgs } = jsonObject;

        if (msgs[0].type === "info") {
          const data = msgs[0].data as GeneralInformation;
          mapScreenData.current.generalInformation.satelliteImageScaleFactor =
            data.satelliteImageScaleFactor;
          mapScreenData.current.generalInformation.shadowFrontierR =
            data.shadowFrontierR;
          mapScreenData.current.generalInformation.shadowFrontierX =
            data.shadowFrontierX;
          mapScreenData.current.generalInformation.shadowFrontierY =
            data.shadowFrontierY;
          //If in record mode
          if (mapScreenData.current.record) {
            //Check if frame already has the info
            if (
              currentFrameDataArray.current.filter(
                (d) => d.type === "General Information"
              ).length < 1
            ) {
              //Check if robot pos. has been updated
              if (
                JSON.stringify(prevGeneralInformation) !== JSON.stringify(data)
              ) {
                //Adding the update to the current frame data
                const newFrameData = {
                  type: "General Information",
                  data: data,
                } as FrameData;
                currentFrameDataArray.current.push(newFrameData);
                //Update prev. state
                prevGeneralInformation = data;
              }
            }
          }
        }
      }

      //Here we update the situation reporting parameters that are shown
      // on the dashboard and map screen
      else if (message.destinationName === "sit/out/robot") {
        const obj = JSON.parse(jsonString);
        mapScreenData.current.conclusion.status =
          obj["status"] === "Success" ? 0 : 1;
        const conclusion = obj["conclusion"];
        mapScreenData.current.conclusion.tornado =
          conclusion["Tornado"] === "no" ? 0 : 1;
        mapScreenData.current.conclusion.earthquake =
          conclusion["Earthquake"] === "no" ? 0 : 1;
        mapScreenData.current.conclusion.vulcano =
          conclusion["Volcano"] === "none"
            ? 0
            : conclusion["Volcano"] === "dormant"
            ? 1
            : 2;
        dashboardScreenData.current.conclusion =
          mapScreenData.current.conclusion;

        //If in record mode
        if (mapScreenData.current.record) {
          //Check if frame already has the info
          if (
            currentFrameDataArray.current.filter((d) => d.type === "Conclusion")
              .length < 1
          ) {
            //Check if situation report conclusion has been updated
            if (
              JSON.stringify(prevConclusion) !==
              JSON.stringify(mapScreenData.current.conclusion)
            ) {
              //Adding the update to the current frame data
              const newFrameData = {
                type: "Conclusion",
                data: mapScreenData.current.conclusion,
              } as FrameData;
              currentFrameDataArray.current.push(newFrameData);
            }
          }
        }
      }

      //com
      else if (message.destinationName === `sim/${config.id}/out/sensors`) {
        const jsonObject = JSON.parse(jsonString) as Messages;
        const { msgs } = jsonObject;

        //console.log(mapScreenData.current.sensors.length);
        mapScreenData.current.sensors = [];

        for (let i = 0; i < msgs.length; i++) {
          if (msgs[i].type === "Wind Sensor") {
            const data = msgs[i].data as WindSensor;
            const speed = Math.sqrt(
              data.wind_vec.x * data.wind_vec.x +
                data.wind_vec.y * data.wind_vec.y
            );
            updateWindSpeed(speed);
            mapScreenData.current.sensors.push(data);
          } else if (msgs[i].type === "Temperature Sensor") {
            const data = msgs[i].data as TemperatureSensor;
            updateTemperature(data.temp);
            mapScreenData.current.sensors.push(data);
          } else if (msgs[i].type === "Seismic Sensor") {
            const data = msgs[i].data as SeismicSensor;
            updateSeismic(data.shake_val);
            mapScreenData.current.sensors.push(data);
          }
        }
        dashboardScreenData.current.sensors = mapScreenData.current.sensors;

        //If in record mode
        if (mapScreenData.current.record) {
          //Check if frame already has the info
          if (
            currentFrameDataArray.current.filter((d) => d.type === "Sensors")
              .length < 1
          ) {
            //Check if robot pos. has been updated
            if (
              JSON.stringify(prevSensors) !==
              JSON.stringify(mapScreenData.current.sensors)
            ) {
              //Adding the update to the current frame data
              const newFrameData = {
                type: "Sensors",
                data: mapScreenData.current.sensors,
              } as FrameData;
              currentFrameDataArray.current.push(newFrameData);
              //Update prev. state
              prevSensors = mapScreenData.current.sensors;
            }
          }
        }
      }

      //com.
      else if (message.destinationName === "tsk/plan") {
        const jsonObject = JSON.parse(jsonString);
        mapScreenData.current.plan = jsonObject;
        dashboardScreenData.current.plan = jsonObject;

        //If in record mode
        if (mapScreenData.current.record) {
          //Check if frame already has the info
          if (
            currentFrameDataArray.current.filter((d) => d.type === "Task Plan")
              .length < 1
          ) {
            //Check if robot pos. has been updated
            if (JSON.stringify(prevPlan) !== JSON.stringify(jsonObject)) {
              //Adding the update to the current frame data
              const newFrameData = {
                type: "Task Plan",
                data: jsonObject,
              } as FrameData;
              currentFrameDataArray.current.push(newFrameData);
              //Update prev. state
              prevPlan = jsonObject;
            }
          }
        }
      }

      //com.
      else if (message.destinationName === "mp/out/general") {
        const jsonObject = JSON.parse(jsonString) as MsgStruct;
        console.log("got new path:");
        console.log(jsonObject);
        if (jsonObject.type === "path") {
          const path = jsonObject.data as any;
          mapScreenData.current.motionPath = path;

          //If in record mode
          if (mapScreenData.current.record) {
            //Check if frame already has the info
            if (
              currentFrameDataArray.current.filter(
                (d) => d.type === "Motion Path"
              ).length < 1
            ) {
              //Check if robot pos. has been updated
              if (JSON.stringify(prevMotionPath) !== JSON.stringify(path)) {
                //Adding the update to the current frame data
                const newFrameData = {
                  type: "Motion Path",
                  data: path,
                } as FrameData;
                currentFrameDataArray.current.push(newFrameData);
                //Update prev. state
                prevMotionPath = path;
              }
            }
          }
        } else {
          console.log("unknown path");
        }
      } else if (message.destinationName === `sim/${config.id}/out/arm`) {
        const jsonObject = JSON.parse(jsonString) as Messages;
        const { msgs } = jsonObject;
        if (msgs[0].type === "arm") {
          const robotarm_data = msgs[0].data as RobotArm;
          mapScreenData.current.robotarm = robotarm_data;
          //If in record mode
          if (mapScreenData.current.record) {
            //Check if frame already has the info
            if (
              currentFrameDataArray.current.filter(
                (d) => d.type === "Robot Arm"
              ).length < 1
            ) {
              //Check if robot pos. has been updated
              if (
                JSON.stringify(prevMotionPath) !== JSON.stringify(robotarm_data)
              ) {
                //Adding the update to the current frame data
                const newFrameData = {
                  type: "Robot Arm",
                  data: robotarm_data,
                } as FrameData;
                currentFrameDataArray.current.push(newFrameData);
                //Update prev. state
                prevRobotArm = robotarm_data;
              }
            }
          }
        }
      }
    } catch (e) {
      console.log(e);
    }
  }

  return (
    <ConfigProvider
      theme={{
        algorithm: theme.darkAlgorithm,
      }}
    >
      <SupabaseContextProvider>
        <RobotContextProvider>
          <OverlayContextProvider>
            <InventoryContextProvider>
              <InstructionContextProvider>
                <MapContextProvider>
                  <Layout>
                    <Routes>
                      <Route
                        path="/"
                        element={<DashboardScreen ref={dashboardScreenData} />}
                      />
                      <Route
                        path="/map"
                        element={
                          <MapScreen
                            source="live"
                            ref={mapScreenData}
                            replay={false}
                            switchAutopilot={switchAutopilot}
                            switchRecord={switchRecord}
                            setHoveredCoordinate={setHoveredCoordinate}
                          />
                        }
                      />
                      <Route path="/replay/:id" element={<ReplayScreen />} />
                      <Route path="/mobile" element={<MobileLandingScreen />} />
                      <Route
                        path="/mobile/joystick"
                        element={<JoystickScreen />}
                      />
                    </Routes>
                  </Layout>
                </MapContextProvider>
              </InstructionContextProvider>
            </InventoryContextProvider>
          </OverlayContextProvider>
        </RobotContextProvider>
      </SupabaseContextProvider>
    </ConfigProvider>
  );
};

export default Navigation;
