import { useEffect, useRef, useState } from "react";
import styles from "./styles/ReplayScreen.module.scss";
import ReplayController from "./components/ReplayController";
import MapScreen from "../map/MapScreen";
import { initReplayScreenData, MapScreenData } from "../models/Data";
import { Frame } from "../models/Replay";
import supabase from "../supabase/Supabase";
import { blobToArrayBuffer } from "../functions/Supabase";
import { decode } from "messagepack";
import Pako from "pako";
import { useParams } from "react-router-dom";
import LoadingScreen from "../components/LoadingScreen";

interface ReplayControllerData {
  frame: number;
  isPlaying: boolean;
  size: number;
}

const ReplayScreen = () => {
  const id = useParams().id || "-1";
  const [loading, setLoading] = useState<boolean>(true);
  const frameArray = useRef<Frame[]>([]);
  const mapScreenData = useRef<MapScreenData>(initReplayScreenData);
  const controllerRef = useRef<ReplayControllerData>({
    frame: 0,
    isPlaying: false,
    size: 1,
  });
  const [speed, setSpeed] = useState<number>(1.0);

  const updateFrame = (newValue: number) => {
    controllerRef.current.frame = newValue;
  };

  const changePlay = (b: boolean) => {
    controllerRef.current.isPlaying = b;
  };

  const changeSpeed = (speed: number) => {
    setSpeed(speed);
  };

  useEffect(() => {
    const interval = setInterval(() => {
      if (frameArray.current.length === 0) return;
      //If the replay is set to playing
      if (controllerRef.current.isPlaying) {
        //If there are frames left
        if (frameArray.current.length > controllerRef.current.frame) {
          //Increase frame count
          controllerRef.current.frame = controllerRef.current.frame + 1;
        } else {
          //If reached end of demo, set the player to pause
          controllerRef.current.isPlaying = false;
        }
      }

      //Loop through the data in a frame, and update the map ref.
      for (let j = 0; j < controllerRef.current.frame; j++) {
        const currentFrame = frameArray.current[j];
        for (let i = 0; i < currentFrame.data.length; i++) {
          const currentData = currentFrame.data[i];
          if (currentData.type === "Robot")
            mapScreenData.current.robot = currentData.data;
          else if (currentData.type === "Autopilot")
            mapScreenData.current.autopilot = currentData.data;
          else if (currentData.type === "Lidar Objects")
            mapScreenData.current.lidarObjects = currentData.data;
          else if (currentData.type === "General Information")
            mapScreenData.current.generalInformation = currentData.data;
          else if (currentData.type === "Conclusion")
            mapScreenData.current.conclusion = currentData.data;
          else if (currentData.type === "Sensors")
            mapScreenData.current.sensors = currentData.data;
          else if (currentData.type === "Task Plan")
            mapScreenData.current.plan = currentData.data;
          else if (currentData.type === "Motion Path")
            mapScreenData.current.motionPath = currentData.data;
          else if (currentData.type === "Situation Report")
            mapScreenData.current.situationReport = currentData.data;
          else if (currentData.type === "Robot Arm")
            mapScreenData.current.robotarm = currentData.data;
        }
      }
    }, 30 / speed);

    return () => clearInterval(interval);
  }, [mapScreenData, frameArray, controllerRef, speed]);

  async function loadDemo() {
    if (id === "-1") return;
    //TODO: CHECK IF THE CURRENT DEMO IS ALREADY LOADED USING ID INSTEAD OF LOAD AGAIN
    if (frameArray.current.length > 0) return;
    console.log("trying to download...");
    try {
      const { data, error } = await supabase.storage
        .from("replay")
        .download("demo_" + id + ".txt");
      if (error) {
        throw error;
      }
      const arrayBuffer = (await blobToArrayBuffer(data)) as ArrayBuffer;
      const decompressedMsg = decode(Pako.inflate(arrayBuffer)) as Frame[];
      frameArray.current = decompressedMsg;
      controllerRef.current.size = frameArray.current.length;
      console.log(frameArray.current);
      console.log("demo loaded");
      setLoading(false);
    } catch (error: any) {
      console.log("Error loading demo: ", error.message);
    }
  }

  loadDemo();

  if (loading) return <LoadingScreen />;

  return (
    <div className={styles.container}>
      <div
        style={{
          width: "100%",
          height: "100%",
          backgroundColor: "black",
        }}
      >
        <MapScreen
          source="demo"
          ref={mapScreenData}
          replay={true}
          switchAutopilot={undefined}
          switchRecord={undefined}
          setHoveredCoordinate={undefined}
        />
      </div>
      <div
        style={{
          position: "absolute",
          bottom: 30,
          zIndex: 10,
          background: "rgba(0,0,0,0.9)",
          borderRadius: 30,
          width: "60%",
        }}
      >
        <ReplayController
          ref={controllerRef}
          updateFrame={updateFrame}
          changePlay={changePlay}
          changeSpeed={changeSpeed}
        />
      </div>
    </div>
  );
};

export default ReplayScreen;
