import React, { useEffect, useMemo, useState } from "react";
import { MQTTConfig } from "../models/MQTT";
import Paho from "paho-mqtt";
import { encode } from "messagepack";
import Pako from "pako";
import { Buffer } from "buffer";

//The members of the interface
interface State {
  client: Paho.Client | undefined;
  config: MQTTConfig;
  topics: string[];
  setConfig: (c: MQTTConfig) => void;
  setTopics: (sa: string[]) => void;
  send: (topic: string, msg: any) => void;
}

//Creating a initial state for the context
const initialState: State = {
  client: undefined,
  config: {
    id: 0,
    host: "tharsis.oru.se",
    port: 8884,
    clientId: "mqtt_ui_qwerty",
    userName: "usr_if",
    password: "usr_if23",
    keepAliveInterval: 4000,
    useSSL: true,
    cleanSession: true,
    compression: "zlib",
    messagepack: true,
  },
  topics: [],
  setConfig: () => {},
  setTopics: () => {},
  send: () => {},
};

const MQTTContext = React.createContext<State>(initialState);

interface Props {
  children: React.ReactNode;
}

export const MQTTContextProvider = ({ children }: Props) => {
  //This is the MQTT client
  const [client, setClient] = useState<Paho.Client | undefined>(
    initialState.client
  );

  //This is the config for the MQTT client
  const [config, setConfig] = useState<MQTTConfig>(initialState.config);

  //This is the topics that the MQTT client will subscribe to
  const [topics, setTopics] = useState<string[]>(initialState.topics);

  useEffect(() => {
    setTopics([
      `sim/${config.id}/out/robotpos`,
      `sit/out/lidar_ui`,
      `sim/${config.id}/out/general`,
      `sit/out/robot`,
      `sim/${config.id}/out/image`,
      `sim/${config.id}/out/sensors`,
      `tsk/plan`,
      `mp/out/general`,
      `sim/${config.id}/out/arm`,
      `sit/out/grids`,
      `sim/${config.id}/out/info`,
    ]);
  }, [config]);

  //This useEffect checks if the config changes, if it does it will reconnect
  // with the new config details
  useEffect(() => {
    try {
      //Checks if the client is initiated
      // if not, sets a new client
      if (!client) {
        setClient(new Paho.Client(config.host, config.port, config.clientId));
        return;
      }

      //Connects the client to the server if it is initiated
      client.connect({
        //If the client succeded with connecting to the server
        // it will console log 'connected' and subscribe to the topics
        onSuccess: () => {
          console.log("connected");
          topics.forEach((t) => client.subscribe(t));
        },

        //Connecting using parameters from config
        userName: config.userName,
        password: config.password,
        keepAliveInterval: config.keepAliveInterval,
        useSSL: config.useSSL,
        cleanSession: config.cleanSession,
      });
    } catch (e) {
      console.log(e);
    }
  }, [client, config, topics]);

  //This send function sends messages with mess
  const send = (topic: string, msg: any) => {
    if (!client) {
      return;
    }
    try {
      var message = msg;

      //If messagepack true in the config the message will be encoded with it
      if (config.messagepack) message = encode(message);

      //If zlib selected as compression in config
      //if (config.compression === "zlib") ;

      //If gzip selected as compression in config
      if (config.compression === "gzip") Pako.gzip(message, { level: 9 });

      //Putting the message into a buffer and sends it to
      // the inputed topic
      const buffer = Buffer.from(Pako.deflate(encode(msg), { level: 9 }));
      let pahoMessage = new Paho.Message(buffer);
      pahoMessage.destinationName = topic;
      client.send(pahoMessage);
    } catch (e) {
      console.log(e);
    }
  };

  const value: State = useMemo(() => {
    return {
      client,
      config,
      topics,
      setConfig,
      setTopics,
      send,
    };
  }, [client, config, topics]);

  return <MQTTContext.Provider value={value}>{children}</MQTTContext.Provider>;
};

export default MQTTContext;
