import moment from "moment";
import React, { useEffect, useMemo, useRef, useState } from "react";
import {
  ActivityLog,
  AutopilotLog,
  BatteryLog,
  SpeedLog,
} from "../models/Supabase";
import { ChartData } from "../models/Supabase";
import supabase from "../supabase/Supabase";

interface State {
  activityLog: ActivityLog[];
  batteryLog: BatteryLog[];
  speedLog: SpeedLog[];
  autopilotLog: AutopilotLog[];
  windSpeed: number[];
  temperature: number[];
  addToActivityLog: (a: ActivityLog) => void;
  addToBatteryLog: (v: number) => void;
  addToSpeedLog: (v: number) => void;
  addToAutopilotLog: (v: number) => void;
  setWindSpeed: (s: number[]) => void;
  setTemperature: (s: number[]) => void;
}
const initialState: State = {
  activityLog: [],
  batteryLog: [],
  speedLog: [],
  autopilotLog: [],
  windSpeed: [0, 0],
  temperature: [0, 0],
  addToActivityLog: () => {},
  addToBatteryLog: () => {},
  addToSpeedLog: () => {},
  addToAutopilotLog: () => {},
  setWindSpeed: () => {},
  setTemperature: () => {},
};

const SupabaseContext = React.createContext<State>(initialState);

interface Props {
  children: React.ReactNode;
}

export const SupabaseContextProvider = ({ children }: Props) => {
  const [activityLog, setActivityLog] = useState<ActivityLog[]>(
    initialState.activityLog
  );
  const [batteryLog, setBatteryLog] = useState<BatteryLog[]>(
    initialState.batteryLog
  );
  const [speedLog, setSpeedLog] = useState<SpeedLog[]>(initialState.speedLog);

  const [autopilotLog, setAutopilotLog] = useState<AutopilotLog[]>(
    initialState.autopilotLog
  );

  const [windSpeed, setWindSpeed] = useState<number[]>(initialState.windSpeed);
  const [temperature, setTemperature] = useState<number[]>(
    initialState.temperature
  );

  const getChartData = async () => {
    try {
      //SET LOADING TRUE HERE
      const { data, error, status } = await supabase.from("ChartData").select();

      if (error && status !== 406) {
        throw error;
      }

      if (data) {
        if (data.length === 0) {
          setAutopilotLog([]);
          setBatteryLog([]);
          setSpeedLog([]);
        }
        const uData = data as unknown;
        let tempData = uData as ChartData[];
        let tempAutopilot = [] as AutopilotLog[];
        let tempBattery = [] as BatteryLog[];
        let tempSpeed = [] as SpeedLog[];
        for (let i = 0; i < tempData.length; i++) {
          if (tempData[i].type === "autopilot") {
            tempAutopilot.push({
              id: tempData[i].id,
              isOn: tempData[i].value,
              time: moment(tempData[i].created_at).utc().format("HH:mm:ss"),
            });
          } else if (tempData[i].type === "battery") {
            tempBattery.push({
              id: tempData[i].id,
              percentage: tempData[i].value,
              time: moment(tempData[i].created_at).utc().format("HH:mm:ss"),
            });
          } else if (tempData[i].type === "speed") {
            tempSpeed.push({
              id: tempData[i].id,
              speed: tempData[i].value,
              time: moment(tempData[i].created_at).utc().format("HH:mm:ss"),
            });
          }
        }
        setAutopilotLog(tempAutopilot);
        setBatteryLog(tempBattery);
        setSpeedLog(tempSpeed);
        //console.log(data);
      }
    } catch (error: any) {
      console.log(error.message);
    } finally {
      //SET LOADING FALSE HERE
    }
  };

  const postChartData = async (type: string, value: number) => {
    try {
      const { error } = await supabase
        .from("ChartData")
        .insert([{ value: value, type: type, created_at: moment() }]);
      getChartData();
      if (error) {
        throw error;
      }
    } catch (error: any) {
      console.log(error.message);
    } finally {
      //setLoading(false)
    }
  };

  const getActivityLog = async () => {
    try {
      //SET LOADING TRUE HERE
      const { data, error, status } = await supabase
        .from("ActivityLog")
        .select();

      if (error && status !== 406) {
        throw error;
      }

      if (data) {
        if (data.length === 0) {
          setActivityLog([]);
        }
        const uData = data as unknown;
        let tempData = uData as ActivityLog[];
        setActivityLog(tempData);
        //console.log(tempData);
      }
    } catch (error: any) {
      console.log(error.message);
    } finally {
      //SET LOADING FALSE HERE
    }
  };

  const postActivityLog = async (type: string, description: string) => {
    try {
      const { error } = await supabase
        .from("ActivityLog")
        .insert([
          { type: type, description: description, created_at: moment() },
        ]);
      getActivityLog();
      if (error) {
        throw error;
      }
    } catch (error: any) {
      console.log(error.message);
    } finally {
      //setLoading(false)
    }
  };

  useEffect(() => {
    getChartData();
    getActivityLog();
  }, []);

  const prevTime = useRef<number>(moment().unix());

  useEffect(() => {
    if (moment().unix() - prevTime.current < 2) {
      return;
    } else {
      prevTime.current = moment().unix();
    }
    /*
    if (message === undefined) return;
    var failed = false;
    const msgString = message.message as string;
    let rawData = msgString as any | undefined;
    console.log(rawData);

    try {
      rawData = JSON.parse(rawData);
      //console.log("[ROBOT.TSX]: Parsed JSON: " + rawData);
    } catch (e) {
      //console.log("[ROBOT.TSX]: Error, failed to parse JSON: " + e);
      failed = true;
    }
    if (!rawData || failed) return;
    const formattedData = rawData as Messages;
    const { msgs } = formattedData;
    if (!msgs) {
      //console.log("[ROBOT.TSX]: Error, invalid data.");
      return;
    }

    for (let i = 0; i < msgs.length; i++) {
      if (msgs[i].type === "Wind Sensor") {
        const data = msgs[i].data as WindSensor;
        const speed = Math.sqrt(
          data.wind_vec.x * data.wind_vec.x + data.wind_vec.y * data.wind_vec.y
        );
        let tempWindSpeed = [...windSpeed];
        const a = tempWindSpeed[0];
        tempWindSpeed = [speed, a];
        setWindSpeed(tempWindSpeed);
      }
      if (msgs[i].type === "Temperature Sensor") {
        const data = msgs[i].data as TemperatureSensor;
        let tempTemperature = [...temperature];
        const a = tempTemperature[0];
        tempTemperature = [data.temp, a];
        setTemperature(tempTemperature);
      }
    } */
  }, [windSpeed, temperature]);

  const value: State = useMemo(() => {
    return {
      activityLog,
      batteryLog,
      speedLog,
      autopilotLog,
      windSpeed,
      temperature,
      addToActivityLog: (a: ActivityLog) => {
        let newActivity = a;
        if (activityLog.length !== 0) {
          const prev = activityLog[activityLog.length - 1];
          newActivity.id = prev.id + 1;
        } else {
          newActivity.id = 0;
        }
        postActivityLog(newActivity.type, newActivity.description);
        setActivityLog((prevState) => [...prevState, newActivity]);
      },
      addToBatteryLog: (v: number) => {
        let newBattery = {} as BatteryLog;
        if (batteryLog.length !== 0) {
          const prev = batteryLog[batteryLog.length - 1];
          newBattery.id = prev.id + 1;
        } else {
          newBattery.id = 0;
        }
        newBattery.percentage = v;
        newBattery.time = moment().format("YYYY-MM-DD HH:mm:ss");
        postChartData("battery", v);
        setBatteryLog((prevState) => [...prevState, newBattery]);
      },
      addToAutopilotLog: (v: number) => {
        let newAutopilot = {} as AutopilotLog;
        if (autopilotLog.length !== 0) {
          const prev = autopilotLog[autopilotLog.length - 1];
          newAutopilot.id = prev.id + 1;
        } else {
          newAutopilot.id = 0;
        }
        newAutopilot.isOn = v;
        newAutopilot.time = moment().format("YYYY-MM-DD HH:mm:ss");
        postChartData("autopilot", v);
        setAutopilotLog((prevState) => [...prevState, newAutopilot]);
      },
      addToSpeedLog: (v: number) => {
        let newSpeed = {} as SpeedLog;
        if (autopilotLog.length !== 0) {
          const prev = speedLog[speedLog.length - 1];
          newSpeed.id = prev.id + 1;
        } else {
          newSpeed.id = 0;
        }
        newSpeed.speed = v;
        newSpeed.time = moment().format("YYYY-MM-DD HH:mm:ss");
        postChartData("speed", v);
        setSpeedLog((prevState) => [...prevState, newSpeed]);
      },
      setWindSpeed,
      setTemperature,
    };
  }, [activityLog, batteryLog, speedLog, autopilotLog, windSpeed, temperature]);

  return (
    <SupabaseContext.Provider value={value}>
      {children}
    </SupabaseContext.Provider>
  );
};

export default SupabaseContext;
